import numpy as np

def two_dim_vertical_second_deriv(numangle, numalt, factor=1):
    """
    Calculates the second derivatvie Tikhonov regularization matrix for a two dimensional uniform grid.
    The matrix is calculated assuming that the measurement vector is constructed with altitude being the leading
    dimension

    Parameters
    ----------
    numangle : scalar
        The number of angular grid points
    numalt : scalar
        The number of altitude grid points
    factor : scalar or length numalt, optional
        If scalar, the resulting matrix is multiplied by this value.  If a vector of length numalt, then each altitude
        level is multiplied by its corresponding factor.  Default is 1
    """

    n = numalt*numangle
    gamma = np.zeros((n, n))
    for idangle in range(0, numangle):
        for idalt in range(1, numalt-1):
            if np.shape(factor) != ():
                mfactor = factor[idalt]
            else:
                mfactor = factor
            idx = idangle*numalt + idalt
            gamma[idx, idx-1] = -1/4 * mfactor
            gamma[idx, idx] = 1/2 * mfactor
            gamma[idx, idx+1] = -1/4 * mfactor

    return gamma
